@@ -2,7 +2,7 @@
 
   VSEARCH: a versatile open source tool for metagenomics
 
-  Copyright (C) 2014-2020, Torbjorn Rognes, Frederic Mahe and Tomas Flouri
+  Copyright (C) 2014-2021, Torbjorn Rognes, Frederic Mahe and Tomas Flouri
   All rights reserved.
 
   Contact: Torbjorn Rognes <torognes@ifi.uio.no>,
@@ -1197,12 +1197,17 @@ void cluster(char * dbname,
     }
 
 
-  /* Sort clusters */
+  /* Sort sequences in clusters by their abundance or ordinal number */
   /* Sequences in same cluster must always come right after each other. */
   /* The centroid sequence must be the first in each cluster. */
 
   progress_init("Sorting clusters", clusters);
-  qsort(clusterinfo, seqcount, sizeof(clusterinfo_t), compare_byclusterno);
+  if (opt_clusterout_sort)
+    qsort(clusterinfo, seqcount, sizeof(clusterinfo_t),
+          compare_byclusterabundance);
+  else
+    qsort(clusterinfo, seqcount, sizeof(clusterinfo_t),
+          compare_byclusterno);
   progress_done();
 
   progress_init("Writing clusters", seqcount);
@@ -1237,7 +1242,9 @@ void cluster(char * dbname,
                                 cluster_abundance[clusterno],
                                 clusterno+1,
                                 -1.0,
-                                -1, -1, 0, 0.0);
+                                -1,
+                                opt_clusterout_id ? clusterno : -1,
+                                0, 0.0);
 
           if (opt_uc)
             fprintf(fp_uc, "C\t%d\t%" PRId64 "\t*\t*\t*\t*\t*\t%s\t*\n",
@@ -1332,15 +1339,6 @@ void cluster(char * dbname,
         }
     }
 
-  if (opt_clusterout_sort)
-    {
-      /* Optionally sort clusters by abundance */
-      progress_init("Sorting clusters by abundance", clusters);
-      qsort(clusterinfo, seqcount, sizeof(clusterinfo_t),
-            compare_byclusterabundance);
-      progress_done();
-    }
-
   if (opt_msaout || opt_consout || opt_profile)
     {
       int msa_target_count = 0;
