@@ -2,7 +2,7 @@
 
   VSEARCH: a versatile open source tool for metagenomics
 
-  Copyright (C) 2014-2020, Torbjorn Rognes, Frederic Mahe and Tomas Flouri
+  Copyright (C) 2014-2021, Torbjorn Rognes, Frederic Mahe and Tomas Flouri
   All rights reserved.
 
   Contact: Torbjorn Rognes <torognes@ifi.uio.no>,
@@ -204,9 +204,10 @@ void rehash(struct bucket * * hashtableref, int64_t alloc_clusters)
   * hashtableref = new_hashtable;
 }
 
-
-void derep_fulllength()
+void derep(char * input_filename, bool use_header)
 {
+  /* dereplicate full length sequences, optionally require identical headers */
+
   show_rusage();
 
   FILE * fp_output = 0;
@@ -226,7 +227,7 @@ void derep_fulllength()
         fatal("Unable to open output (uc) file for writing");
     }
 
-  fastx_handle h = fastx_open(opt_derep_fulllength);
+  fastx_handle h = fastx_open(input_filename);
 
   show_rusage();
 
@@ -281,7 +282,7 @@ void derep_fulllength()
   char * rc_seq_up = (char*) xmalloc(alloc_seqlen + 1);
 
   char * prompt = 0;
-  if (xsprintf(& prompt, "Dereplicating file %s", opt_derep_fulllength) == -1)
+  if (xsprintf(& prompt, "Dereplicating file %s", input_filename) == -1)
     fatal("Out of memory");
 
   progress_init(prompt, filesize);
@@ -368,6 +369,8 @@ void derep_fulllength()
         }
 
       char * seq = fastx_get_sequence(h);
+      char * header = fastx_get_header(h);
+      int64_t headerlen = fastx_get_header_length(h);
 
       /* normalize sequence: uppercase and replace U by T  */
       string_normalize(seq_up, seq, seqlen);
@@ -384,14 +387,21 @@ void derep_fulllength()
         collision when the number of sequences is about 5e9.
       */
 
-      uint64_t hash = HASH(seq_up, seqlen);
+      uint64_t hash_header;
+      if (use_header)
+        hash_header = HASH(header, headerlen);
+      else
+        hash_header = 0;
+
+      uint64_t hash = HASH(seq_up, seqlen) ^ hash_header;
       uint64_t j = hash & hash_mask;
       struct bucket * bp = hashtable + j;
 
       while ((bp->size)
              &&
              ((hash != bp->hash) ||
-              (seqcmp(seq_up, bp->seq, seqlen))))
+              (seqcmp(seq_up, bp->seq, seqlen)) ||
+              (use_header && strcmp(header, bp->header))))
         {
           j = (j+1) & hash_mask;
           bp = hashtable + j;
@@ -402,14 +412,15 @@ void derep_fulllength()
           /* no match on plus strand */
           /* check minus strand as well */
 
-          uint64_t rc_hash = HASH(rc_seq_up, seqlen);
+          uint64_t rc_hash = HASH(rc_seq_up, seqlen) ^ hash_header;
           uint64_t k = rc_hash & hash_mask;
           struct bucket * rc_bp = hashtable + k;
 
           while ((rc_bp->size)
                  &&
                  ((rc_hash != rc_bp->hash) ||
-                  (seqcmp(rc_seq_up, rc_bp->seq, seqlen))))
+                  (seqcmp(rc_seq_up, rc_bp->seq, seqlen)) ||
+                  (use_header && strcmp(header, bp->header))))
             {
               k = (k+1) & hash_mask;
               rc_bp = hashtable + k;
@@ -424,7 +435,6 @@ void derep_fulllength()
             }
         }
 
-      char * header = fastx_get_header(h);
       int abundance = fastx_get_abundance(h);
       int64_t ab = opt_sizein ? abundance : 1;
       sumsize += ab;
@@ -1086,3 +1096,13 @@ void derep_prefix()
   xfree(hashtable);
   db_free();
 }
+
+void derep_fulllength()
+{
+  derep(opt_derep_fulllength, false);
+}
+
+void derep_id()
+{
+  derep(opt_derep_id, true);
+}
