@@ -2,7 +2,7 @@
 
   VSEARCH: a versatile open source tool for metagenomics
 
-  Copyright (C) 2014-2020, Torbjorn Rognes, Frederic Mahe and Tomas Flouri
+  Copyright (C) 2014-2021, Torbjorn Rognes, Frederic Mahe and Tomas Flouri
   All rights reserved.
 
   Contact: Torbjorn Rognes <torognes@ifi.uio.no>,
@@ -84,19 +84,18 @@ void buffer_filter_extend(fastx_handle h,
                           uint64_t len,
                           unsigned int * char_action,
                           const unsigned char * char_mapping,
-                          uint64_t lineno_start)
+                          bool * ok,
+                          char * illegal_char)
 {
   buffer_makespace(dest_buffer, len+1);
 
   /* Strip unwanted characters from the string and raise warnings or
      errors on certain characters. */
 
-  uint64_t lineno = lineno_start;
-
   char * p = source_buf;
   char * d = dest_buffer->data + dest_buffer->length;
   char * q = d;
-  char msg[200];
+  * ok = true;
 
   for(uint64_t i = 0; i < len; i++)
     {
@@ -118,17 +117,9 @@ void buffer_filter_extend(fastx_handle h,
 
         case 2:
           /* fatal character */
-          if ((c>=32) && (c<127))
-            snprintf(msg,
-                     200,
-                     "Illegal character '%c'",
-                     c);
-          else
-            snprintf(msg,
-                     200,
-                     "Illegal unprintable ASCII character no %d",
-                     (unsigned char) c);
-          fastq_fatal(lineno, msg);
+          if (*ok)
+            * illegal_char = c;
+          * ok = false;
           break;
 
         case 3:
@@ -137,7 +128,6 @@ void buffer_filter_extend(fastx_handle h,
 
         case 4:
           /* newline (silently stripped) */
-          lineno++;
           break;
         }
     }
@@ -177,6 +167,10 @@ bool fastq_next(fastx_handle h,
 
   h->lineno_start = h->lineno;
 
+  char msg[200];
+  bool ok = true;
+  char illegal_char = 0;
+
   uint64_t rest = fastx_file_fill_buffer(h);
 
   /* check end of file */
@@ -221,8 +215,6 @@ bool fastq_next(fastx_handle h,
       rest -= len;
     }
 
-  uint64_t lineno_seq = h->lineno;
-
   /* read sequence line(s) */
   lf = 0;
   while (1)
@@ -255,18 +247,27 @@ bool fastq_next(fastx_handle h,
                            & h->sequence_buffer,
                            h->file_buffer.data + h->file_buffer.position,
                            len,
-                           char_fq_action_seq, char_mapping, lineno_seq);
+                           char_fq_action_seq, char_mapping,
+                           & ok, & illegal_char);
       h->file_buffer.position += len;
       rest -= len;
+
+      if (!ok)
+        {
+          if ((illegal_char >= 32) && (illegal_char < 127))
+            snprintf(msg,
+                     200,
+                     "Illegal sequence character '%c'",
+                     illegal_char);
+          else
+            snprintf(msg,
+                     200,
+                     "Illegal sequence character (unprintable, no %d)",
+                     (unsigned char) illegal_char);
+          fastq_fatal(h->lineno - (lf ? 1 : 0), msg);
+        }
     }
 
-#if 0
-  if (h->sequence_buffer.length == 0)
-    fastq_fatal(lineno_seq, "Empty sequence line");
-#endif
-
-  uint64_t lineno_plus = h->lineno;
-
   /* read + line */
 
   /* skip + character */
@@ -319,13 +320,11 @@ bool fastq_next(fastx_handle h,
         plusline_invalid = 1;
     }
   if (plusline_invalid)
-    fastq_fatal(lineno_plus,
+    fastq_fatal(h->lineno - (lf ? 1 : 0),
                 "'+' line must be empty or identical to header");
 
   /* read quality line(s) */
 
-  uint64_t lineno_qual = h->lineno;
-
   lf = 0;
   while (1)
     {
@@ -354,22 +353,38 @@ bool fastq_next(fastx_handle h,
           len = lf - (h->file_buffer.data + h->file_buffer.position) + 1;
           h->lineno++;
         }
+
       buffer_filter_extend(h,
                            & h->quality_buffer,
                            h->file_buffer.data + h->file_buffer.position,
                            len,
-                           char_fq_action_qual, chrmap_identity, lineno_qual);
+                           char_fq_action_qual, chrmap_identity,
+                           & ok, & illegal_char);
       h->file_buffer.position += len;
       rest -= len;
+
+      /* break if quality line already too long */
+      if (h->quality_buffer.length > h->sequence_buffer.length)
+        break;
+
+      if (!ok)
+        {
+          if ((illegal_char >= 32) && (illegal_char < 127))
+            snprintf(msg,
+                     200,
+                     "Illegal quality character '%c'",
+                     illegal_char);
+          else
+            snprintf(msg,
+                     200,
+                     "Illegal quality character (unprintable, no %d)",
+                     (unsigned char) illegal_char);
+          fastq_fatal(h->lineno - (lf ? 1 : 0), msg);
+        }
     }
 
-#if 0
-  if (h->quality_buffer.length == 0)
-    fastq_fatal(lineno_seq, "Empty quality line");
-#endif
-
   if (h->sequence_buffer.length != h->quality_buffer.length)
-    fastq_fatal(lineno_qual,
+    fastq_fatal(h->lineno - (lf ? 1 : 0),
                 "Sequence and quality lines must be equally long");
 
   fastx_filter_header(h, truncateatspace);
@@ -483,6 +498,9 @@ void fastq_print_general(FILE * fp,
                                   xee);
     }
 
+  if (opt_label_suffix)
+    fprintf(fp, "%s", opt_label_suffix);
+
   if (opt_sizeout && (abundance > 0))
     fprintf(fp, ";size=%u", abundance);
 
