@@ -2,7 +2,7 @@
 
   VSEARCH: a versatile open source tool for metagenomics
 
-  Copyright (C) 2014-2020, Torbjorn Rognes, Frederic Mahe and Tomas Flouri
+  Copyright (C) 2014-2021, Torbjorn Rognes, Frederic Mahe and Tomas Flouri
   All rights reserved.
 
   Contact: Torbjorn Rognes <torognes@ifi.uio.no>,
@@ -323,6 +323,13 @@ void sintax_query(int64_t t)
   int boot_count[2];
   unsigned int best_count[2];
 
+  best_count[0] = 0;
+  best_count[1] = 0;
+  best_seqno[0] = 0;
+  best_seqno[1] = 0;
+  boot_count[0] = 0;
+  boot_count[1] = 0;
+
   int qseqlen = si_plus[t].qseqlen;
   char * query_head = si_plus[t].query_head;
 
@@ -344,10 +351,6 @@ void sintax_query(int64_t t)
 
       /* perform 100 bootstraps */
 
-      best_count[s] = 0;
-      best_seqno[s] = 0;
-      boot_count[s] = 0;
-
       if (kmersamplecount >= subset_size)
         {
           for (int i = 0; i < bootstrap_count ; i++)
