@@ -2,7 +2,7 @@
 
   VSEARCH: a versatile open source tool for metagenomics
 
-  Copyright (C) 2014-2020, Torbjorn Rognes, Frederic Mahe and Tomas Flouri
+  Copyright (C) 2014-2021, Torbjorn Rognes, Frederic Mahe and Tomas Flouri
   All rights reserved.
 
   Contact: Torbjorn Rognes <torognes@ifi.uio.no>,
@@ -60,7 +60,7 @@
 
 #include "vsearch.h"
 
-static struct sortinfo_s
+static struct sortinfo_size_s
 {
   unsigned int size;
   unsigned int seqno;
@@ -68,8 +68,8 @@ static struct sortinfo_s
 
 int sortbysize_compare(const void * a, const void * b)
 {
-  struct sortinfo_s * x = (struct sortinfo_s *) a;
-  struct sortinfo_s * y = (struct sortinfo_s *) b;
+  struct sortinfo_size_s * x = (struct sortinfo_size_s *) a;
+  struct sortinfo_size_s * y = (struct sortinfo_size_s *) b;
 
   /* highest abundance first, then by label, otherwise keep order */
 
@@ -108,7 +108,8 @@ void sortbysize()
 
   progress_init("Getting sizes", dbsequencecount);
 
-  sortinfo = (struct sortinfo_s*) xmalloc(dbsequencecount * sizeof(sortinfo_s));
+  sortinfo = (struct sortinfo_size_s*)
+    xmalloc(dbsequencecount * sizeof(sortinfo_size_s));
 
   int passed = 0;
 
@@ -130,7 +131,7 @@ void sortbysize()
   show_rusage();
 
   progress_init("Sorting", 100);
-  qsort(sortinfo, passed, sizeof(sortinfo_s), sortbysize_compare);
+  qsort(sortinfo, passed, sizeof(sortinfo_size_s), sortbysize_compare);
   progress_done();
 
   double median = 0.0;
