@@ -2,7 +2,7 @@
 
   VSEARCH: a versatile open source tool for metagenomics
 
-  Copyright (C) 2014-2020, Torbjorn Rognes, Frederic Mahe and Tomas Flouri
+  Copyright (C) 2014-2021, Torbjorn Rognes, Frederic Mahe and Tomas Flouri
   All rights reserved.
 
   Contact: Torbjorn Rognes <torognes@ifi.uio.no>,
@@ -70,13 +70,12 @@
 #define exp10(x) (pow(10.0,(x)))
 #endif
 
-#define MD5_DIGEST_LENGTH 16
 #define SHA_DIGEST_LENGTH SHA1_DIGEST_SIZE
 
-#define LEN_DIG_MD5 MD5_DIGEST_LENGTH
+constexpr int MD5_DIGEST_LENGTH {16};
 #define LEN_DIG_SHA1 SHA_DIGEST_LENGTH
 
-#define LEN_HEX_DIG_MD5 (2*LEN_DIG_MD5+1)
+constexpr int LEN_HEX_DIG_MD5 {2 * MD5_DIGEST_LENGTH + 1};
 #define LEN_HEX_DIG_SHA1 (2*LEN_DIG_SHA1+1)
 
 void fatal(const char * msg);
